/*
 * Plugin Created by ReadMe <glenjamin[at]gmail.com>
 * Can usually be found in irc://irc.moznet.org/chatzilla
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * (Taken from http://people.freebsd.org/~phk/)
 * ReadMe <glenjamin[at]gmail.com> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return
 * ----------------------------------------------------------------------------
 */

plugin.id = "countdown";

var myprefs = [
["counters", "", ""],
];
var counters;
const msinaweek = 604800000;
const msinaday = 86400000;
const msinahour = 3600000;
const msinaminute = 60000;

plugin.init =
function init(glob)
{
    /* This function will be called when chatzilla first loads the plugin.
     * We initialize some of the plugin object's properties, define a new
     * command and some new prefs, and print out a message.
     */
    plugin.major = 0;
    plugin.minor = 1;
    plugin.version = plugin.major + "." + plugin.minor;
    plugin.description = "Countdown timer to events"
    plugin.prefary = plugin.prefary.concat(myprefs);

}

plugin.disable =
function disable()
{

    client.commandManager.removeCommands(plugin.commands);

    saveCounters();
    display(plugin.id+' plugin disabled');

    return true;
}

plugin.enable =
function enable()
{
    var cmdary =
    [/* comand name     function              flags              usage*/
     ["cdown",          cmdCDown,              null,              "<date> <event>"],
     ["cdshow",         cmdCShow,              null,              "<event>"],
     ["cdsay",          cmdCShow,      CMD_CONSOLE|CMD_NEED_SRV,  "<event>"],
     ["cdpurge",        purgeCounters,         null,              ""],
     ["cdlist",         cmdCList,              null,              "[<search>]"],
     ["cddel",          cmdCDel,               null,              ""],
    ]

    cmdary.stringBundle = client.messageManager.addBundle(plugin.cwd+"default.properties");
    client.commandManager.argTypes["date"] = parseDate;
    client.commandManager.argTypes.__aliasTypes__(["event","search"], "rest");
    plugin.commands = client.commandManager.defineCommands(cmdary);

    if(plugin.prefs['counters']!="") {
        counters = eval(plugin.prefs['counters']);
    } else {
        counters = {};
    }

    purgeCounters();

    display(plugin.id+' plugin enabled');

    return true;
}

/** I/O parsing functions **/

function parseDate(e,name) {

    var ary = e.unparsedData.match (/^(\d{4})\-(\d{1,2})\-(\d{1,2})(?:\s+(\d{1,2})\:(\d{1,2})(?:\:(\d{1,2}))?)?(?:\s+(.*))?$/);
    if (!ary)
        return false;

    --ary[2];
        
    if (!ary[4])
        ary[4] = ary[5] = ary[6] = 0;
    
    if (!ary[6])
        ary[6] = 0;
    
    for (i=1;i<6;++i) {
        if (isNaN(ary[i]))
            return false;
        ary[i] = parseInt(ary[i]);
    }

    e[name] = new Date(ary[1],ary[2],ary[3],ary[4],ary[5],ary[6],0);

    e.unparsedData = arrayHasElementAt(ary, 7) ? ary[7] : "";
    
    return true;
    
}


/** General Util functions **/

function saveCounters() {

    plugin.prefs['counters'] = uneval(counters);
    display('Stored Counters pref updated');

}

function purgeCounters() {

    var now = new Date(); 
    for (evt in counters) {
        var edate = counters[evt].date;
        var theevent = counters[evt].event; 
        
        var until = edate - now;
        if (until < 0) {
            display("\x02"+theevent+"\x02 has passed!");
            delete counters[evt];
        }
    
    }

}

function timestr(ms) {
    numweeks = Math.floor(ms / msinaweek);
    ms -= numweeks*msinaweek;
    weekpl = (numweeks != 1) ? 's' : '' ;
    numdays = Math.floor(ms / msinaday);
    ms -= numdays*msinaday;
    daypl = (numdays != 1) ? 's' : '' ;
    numhours = Math.floor(ms / msinahour);
    ms -= numhours*msinahour;
    hourpl = (numhours != 1) ? 's' : '' ;
    numminutes = Math.round(ms / msinaminute);
    ms -= numminutes*msinaminute;
    minpl = (numminutes != 1) ? 's' : '' ;
    
    return numweeks + " week"+weekpl+", " + numdays + " day"+daypl+", " + numhours + " hour"+hourpl+", and " + numminutes + " minute"+minpl;
}

/** Command Functions **/

function cmdCDown(e) {

    try {
        if (!isinstance(e.date,Date))
            throw('Invalid date');
        var now = new Date();
        if (e.date < now)
            throw('Event passed');
         
        counters[e.event.toLowerCase()] = {'date': e.date, 'event': e.event};
    
        display("\x02"+e.event+"\x02 Will occur on "+e.date.toLocaleString());
        saveCounters();
    } catch(ex) {
        display(ex,'ERROR');
    }
}
function cmdCList(e) {

		if(e.search) {
	    searchstring = new RegExp(e.search);
	    for (var evt in counters) {
	        if(evt.match(searchstring)) {
	            var shown = true;
	            dispatch("cdshow", {'event': evt});
	        }
	    }
	  } else {
	  	for (var evt in counters) {
        var shown = true;
        dispatch("cdshow", {'event': evt});
	    }
	  }
    if (!shown) display('No Counters Found','ERROR');
}
function cmdCShow(e) {

    try {
        if  (!(e.event in counters))
            throw('Event does not exist');
        
        var now = new Date(); 
        var edate = counters[e.event.toLowerCase()].date;
        var theevent = counters[e.event.toLowerCase()].event; 
    
        var until = edate - now;
        if (until < 0) {
            display('Event has passed!');
            delete counters[e.event];  
        }
    
        var str = timestr(until);
        str = "\x02"+theevent+"\x02 will occur in \x02"+str;
        
        if (e.commandText == 'cdshow')
            display(str);
        else
            dispatch("say "+str);
        
    } catch(ex) {
        display(ex,'ERROR');
    }

}

function cmdCDel(e) {
	counters = {};
	plugin.prefs['counters'] = "";
	display('Counters deleted')
}